package utils.GUI;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;


/****************************************************************************
 * A class which mimics an uneditable TextField for displaying double\
 * (or float) values;  It automatically truncates the given value
 * to a number of significant figures(given at construction). In addition
 * it switches between displaying the number in floating point notation
 * and scientific notation.
 *********************************************************************/

public class DoubleTextDisplay extends Canvas{
  final static int margin=4;
  final static String EMPTY=new String("");

  int sig; // number of significant figures to display;
  int abs_size;  // maximal size(in char) of the base part


  FontMetrics abs_fm; // font metrics for the absica
  FontMetrics exp_fm; // font metrics for the exponent
  Font exp_fn;
  double value;
  String abs;   // string value of absica
  String exp;  // string value of exponent
  
  int abs_x,abs_y;  // x,y positions of the absica string
  int exp_x,exp_y;  // x.y positions of the exponent string



  /**
   * @param v -- initial value to display
   * @param sigfig -- number of significant figures
   * @param size   -- maximal number of characters to display in Floating point
   **/

  public DoubleTextDisplay(double v,int sigfig,int size){
    super();
    abs_size=Math.max(sigfig+3,size);
    
    sig=sigfig;
    // leave off rest until we have a Font
    setValue(v);
  }

  public DoubleTextDisplay(double v,int sigfig){
    this(v,sigfig,0);
  }

  public DoubleTextDisplay(int sigfig){
    this(0.0,sigfig,0);
  }

  public DoubleTextDisplay(int sigfig,int size){
   this(0.0,sigfig,size);
  }



  void measure(Font abs_fn){
    // determines fonts and locations to draw the strings
    abs_fm=getFontMetrics(abs_fn);
    exp_fn=new Font( abs_fn.getName(), // same style
		     abs_fn.getStyle(),
		     3*abs_fn.getSize()/4 // but smaller
		     );
    exp_fm=getFontMetrics(exp_fn);
    abs_x=margin;
    abs_y=margin+abs_fm.getHeight()+exp_fm.getHeight()/2-4;
    exp_y=margin+exp_fm.getHeight();
  }

  public void addNotify(){
    super.addNotify();
    measure( getFont());
    resize( minimumSize());
  }
  public Dimension minimumSize(){
    
    int h=2*margin+abs_fm.getHeight()+exp_fm.getHeight()/2;
    int w=(abs_size+1)*abs_fm.charWidth('0')+4*exp_fm.charWidth('0')+
      2*margin;
    return new Dimension(w,h);
  }

  public Dimension preferredSize(){
    return minimumSize();
  }
  
  public void setFont(Font f){
    super.setFont(f);
    measure(f);
  }

  public void paint(Graphics g){
    Dimension d=size();
    
    g.setColor( getBackground());
    for( int i=0;i<3;++i){
      g.draw3DRect(i,i,d.width-2*i,d.height-2*i,false);
    }
    g.setColor( getForeground());
    g.drawString( abs, abs_x,abs_y);
    
    if( ! EMPTY.equals( exp ) ){
      exp_x=abs_x+abs_fm.stringWidth( abs ) +1 ;
      g.setFont( exp_fn );
      g.drawString( exp , exp_x,exp_y);
    }
  }

  public void setValue(double v){
    value=v;
    
    int sign;
    if( v < 0.0 ){
      sign=-1;
      v=-v;
    }
    else{
      sign=+1;
    }
    int pow;
    if( v > 0.0 ){
      pow= (int)( log10(v) );
      if( v<1.0 ) --pow;
    }
    else pow=0;
    
    if(  pow > sig-2 || pow < -sig+2 ){
      setUpExp(sign,v,pow);
    }
    else setUpFP(sign,v,pow);
    repaint();
  }

  public double getValue(){ return value; }

  void setUpFP(int sign,double value,int pow){
    if( value < 1.0 ) ++pow;
    double factor=ipow(10.0,pow+sig);
    double new_value=( Math.round( value*factor) )/factor;
    
    StringBuffer tmp;
    if( sign < 0 ) tmp=new StringBuffer("-");
    else           tmp=new StringBuffer(" ");
    tmp.append(new_value);
    
    if( tmp.toString().indexOf('E') > -1 || 
	tmp.toString().indexOf('.') > abs_size-1){
      setUpExp(sign,value,pow);
      return;
    }

    while( tmp.length() < abs_size ) tmp.append('0');
    
    tmp.setLength( abs_size );
    abs=tmp.toString();
    exp=EMPTY;
  }
  
  void setUpExp(int sign,double value,int pow){
    value= ( Math.round( value*ipow(10.0, sig-pow-1) ))/ipow(10.0,sig-1);
   
    
    StringBuffer tmp;
    if( sign < 0 ) tmp=new StringBuffer("-");
    else           tmp=new StringBuffer(" ");
  
    tmp.append(value);
    while( tmp.length() < sig+2 ) tmp.append('0');
    tmp.setLength( sig+2 );
    tmp.append("x10");
    abs=tmp.toString();
    exp=String.valueOf(pow);
  }





  static final double LN10=Math.log(10.0);
  

  final double log10(double v){
    return Math.log(v)/LN10;
  }
  final double ipow(double base,int exp){
    if( exp < 0 ) return 1.0/ipow(base,-exp);
    if( exp==0 ) return 1.0;
    if( exp==1 ) return base;
    return ipow( base*base, exp/2 )*ipow(base,exp%2);
  }

}
