package utils.d2.primitive;

import utils.d2.primitive.GraphicsPrimitive;
import utils.d2.primitive.VectorOfPrimitives;
import utils.d2.TwoPerspective;
import java.awt.Graphics;
import java.awt.Color;


/******************************************************************
 * organizes a bunch of GraphicsPrimitives by Color
 ******************************************************************/

public class Palette{
  VectorOfPrimitives[] stuff;
  Color[]            color;

  int  current;
  int max;
  int block;


  /**
   * specifiy initial size, and block size.
   **/
  public Palette(int initial,int bl){
    max=Math.max(initial,2);
    current=0;
    stuff=new VectorOfPrimitives[max];
    color=new Color[max];
    block=Math.max(2,bl);
  }

  public Palette(){
    this(5,5);
  }
  
  /**
   * define a Palette by listing the colors in it.
   * the lower indexed colors will be drawn under the higher indexed
   **/
  public Palette(Color[] hue){
    this(hue.length,5);
    for(int i=0;i<hue.length;++i){
      color[i]=hue[i];
      stuff[i]=new VectorOfPrimitives();
    }
    current=hue.length;
  }


  public int indexOf(Color c){
    for(int i=current-1;i>-1;i-=1){
      if( color[i].equals(c) ) return i;
    }
    return -1;
  }
  public Color colorAt(int color_index){
    return color[color_index];
  }
  
  
  public int n_Colors(){ return current; }
  
  /**
   * returns all the GraphicsPrimitives drawn in the given color
   **/
  public VectorOfPrimitives vectorAt(Color c){
    return stuff[ indexOf(c) ];
  }

  /**
   * returns the number of primitives having this color
   **/
  public int n_Primitives(Color c){
    return stuff[ indexOf(c) ].size();
  }
  
  /**
   * returns the i'th primitive with a given color.
   * the order in which the primitives are stored is currently arbitrary
   **/
  public GraphicsPrimitive primitiveAt(Color c,int i){
    return primitiveAt( indexOf(c),i);
  }

  /**
   * removes all primitives from the palette.
   * but keeps the arrangement of colors in place
   **/
  public void removeAllPrimitives(){
    for( int i=0; i< current; ++i){
      stuff[i].removeAll();
    }
  }
      
  
   /**
   *  returns the index of the color,if it exists; otherwise adds the
   *  color to the end of the Palette, and returns its index
   **/
  public synchronized int ensureColor(Color c){
    int c_index=indexOf(c);
    if( c_index < 0 ){
      c_index=appendColor(c);
    }
    return c_index;
  }
  
  /**
   * removes the color from the Palette.
   * @returns the vector of primitives which were drawn in that color
   **/
  
  public VectorOfPrimitives removeColor(Color c){
    int c_index=indexOf(c);
    if( c_index < 0 ) return null;
    --current;
    shift(c_index,current,+1);  // move the color to the end of the palette
    VectorOfPrimitives tmp=stuff[current];
    stuff[current]=null;
    color[current]=null;
    return tmp;
  }

  /**
   * moves the color to the given index
   **/
  public void moveColor(Color c,int new_index){
    int c_index=indexOf(c);
    if( c_index < new_index ) shift( c_index,new_index,+1);
    if( c_index > new_index ) shift( c_index,new_index,-1);
  }

  

  
    /**
   * inserts the GP with the given color
   **/
  public void add(Color c,GraphicsPrimitive gp){
    int c_index=ensureColor(c);
    stuff[c_index].add(gp);
  }
  public void addSeperately(Color c,VectorOfPrimitives vop){
    int c_index=ensureColor(c);
    stuff[c_index].addSeperately(vop);
  }

  public void add(Palette p){
    for(int i=0;i<p.n_Colors();++i){
      Color hue=p.colorAt(i);
      addSeperately( hue, p.vectorAt(i) );
    }
  }
  



 public void updateDisplay(TwoPerspective p){
    for(int i=0;i<current;++i) stuff[i].updateDisplay(p);
  }

  public void draw(Graphics g){
  
    for(int i=0;i<current;++i){
      g.setColor( color[i]);
      stuff[i].draw(g);
    }
  }
	  
  public void updateAndDraw(TwoPerspective p,Graphics g){
    updateDisplay(p);
    draw(g);
  }

  public void removePrimitive(Color c,GraphicsPrimitive gp){
    int c_index=indexOf(c);
    int gp_index=stuff[c_index].indexOf(gp);
    if( gp_index > -1) stuff[c_index].removeAt(gp_index);
  }


 GraphicsPrimitive primitiveAt(int color_index,int i){
    return stuff[color_index].primitiveAt(i);
  }
  VectorOfPrimitives vectorAt(int color_index){
    return stuff[color_index];
  }

  

 
 
  void augmentCapacity(int increment){
    if( current+increment > max ){
      VectorOfPrimitives[] oldstuff=stuff;
      Color[]             oldcolor=color;
      max+=(increment+block);
      stuff=new VectorOfPrimitives[max];
      color=new Color[max];
      System.arraycopy(oldstuff,0,stuff,0,current);
      System.arraycopy(oldcolor,0,color,0,current);
    }
  }




  int appendColor(Color c){
    augmentCapacity(1);
    color[current]=c;
    stuff[current]=new VectorOfPrimitives();
    ++current;
    return current-1;
  }


  void swap(int i,int j){
    Color c_tmp=color[i];
    VectorOfPrimitives vop_tmp=stuff[i];
    color[i]=color[j]; stuff[i]=stuff[j];
    color[j]=c_tmp;    stuff[j]=vop_tmp;
  }
  
  // moves the stuff(i.e. color, gp's ) at index start index
  // up through the palette to the location given by
  // final_index.  d must be either +1 or -1.
  // d=+1 moves start up to final
  // d=-1 moves start down to final

  void shift(int start_index,int final_index,int d){
    for(int i=start_index; i!=final_index;i+=d){
      swap(i,i+d);
    }
  }



}
